{{
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
Cogject demonstration program - load all cogs through a common section of hub memory
and save program space

To run, first copy all .cog files to an SD card

Minimalist SD card loader program - disable RTC by setting the RTC constants to -1
Take FullDuplexSerial and remove the spin code and save as Cog_FullDuplexSerial.
Compile to a binary and rename with shorter filename serial.cog and copy to SD card
Take FullDuplexSerial and remove the cog code and save as Spin_FullDuplexSerial.
  Comment out cognew and cogstop and replace cognew with okay := rx_head so returns pointer to the parameter list
Add   Serial: "Spin_FullDuplexSerial.spin"    to the OBJ section
Temporarily add VT100 display driver to OBJ section for debugging
Add code to open the file and close it
Add string object
Open file and read file into CogArray
Load cog and run
Split Keyboard.spin into Cog_keyboard and Spin_keyboard
Compile cog_keyboard, save as .binary, rename to keyboard.cog, copy to sd card
Modify spin_keyboard so returns location of the parameter list
Modified load so discards first 24 bits, and now load is in sub object, not the main
Added VT100 code using the vga 80x40 cog loaded off the sd card
Commented out VT100 for the moment otherwise two screen buffers taking too much space
Added VGA cogjects for 80x40, 100x50 and 128x64 text drivers
Added Mouse cogject  - this one needed hardly any rewriting
Added Kye's 320x240 gray scale (or 4 color) video driver
Note that this program is now full - no room left for any code unless something is removed


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}}

CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  _SD_DO = 12
  _SD_CLK = 13
  _SD_DI = 14
  _SD_CS = 15
  _SD_WP = -1 ' -1 ifnot installed.
  _SD_CD = -1 ' -1 ifnot installed.

  _RTC_DAT = -1 ' -1 ifnot installed.
  _RTC_CLK = -1 ' -1 ifnot installed.


OBJ

  fat:     "SD3.01_FATEngine_Tiny.spin" ' tiny version with most PUBs commented and no RTC
 'VT100:   "VT100_80_40"       ' VGA VT100 Terminal Driver used for debugging only as not a cogject 
  Serial:  "Spin_FullDuplexSerial.spin"
  kb :     "Spin_Keyboard.spin"
  vga80  : "Spin_VGA_80x40.spin"
 'vga100 : "Spin_VGA_100x50.spin"    ' commented out to save space for testing other cogjects
  'vga128 : "Spin_VGA_128x64.spin"
  delay  : "Timing.spin"
  mouse  : "Spin_Mouse.spin"
  pix    : "Spin_Kye_160x120.spin"
  ram    : "Spin_Dracblade.spin"
  gray320: "Spin_Kye_320x240.spin"
  trig   : "Spin_Trig.spin"
  
  
 
VAR
    long cogarray               ' recycle the cog space used by fatengine, which happens to be 494 longs

    ' variables used by vga text drivers
      'sync long - written to -1 by VGA driver after each screen refresh
  long  sync
  'screen buffer - could be bytes, but longs allow more efficient scrolling
  long  screen[160*120/4]     ' Biggest is graphics, change for various screen sizes, eg 80x40 text is 80*40/4
  'row colors
  word  colors[64]              ' text vga display colors max 64 rows, one color per row
  'cursor control bytes
  byte  cx0, cy0, cm0, cx1, cy1, cm1   ' mouse variables
  
    
PUB Main | i,j,cols,rows,chrs
    
  fat.FATEngineStart(_SD_DO, _SD_CLK, _SD_DI, _SD_CS, _SD_WP, _SD_CD, _RTC_DAT, _RTC_CLK, -1)
  cogarray := fat.addressofcog                          ' get location of the fat cog array
  fat.mountPartition(0)                                 ' mount the sd card
  'ifnot(fat.partitionMounted)                          ' commented out - if the sd card isn't working, can't load a cog to be able to display this
  '  printstringcr(string("Card Not Mounted "))
  ReadCogFile(string("Serial.Cog"))                     ' cog 2
  serial.start(cogarray,31,30,0,38400)                  ' start the cog at this baud rate

  ReadCogFile(string("Keyboard.Cog"))                   ' cog 3
  kb.start(cogarray,26,27)                              'start the keyboard

  ' start graphics vga driver

  ReadCogFile(string("VGAGraph.Cog"))                   ' cog 4
  pix.PIXEngineStart(cogarray, @screen, 2)              ' pin group 2 for VGA
  'pix.plotpixel(@screen, %00001100,30,30)               ' plot a pixel
  pix.displayclear(@screen) 
  repeat i from 0 to 500                                ' put some dots on the screen
      pix.plotPixel(@screen, pix.displayColor( ((||(cnt?)) // 4), ((||(cnt?)) // 4), ((||(cnt?)) // 4)), ((||(cnt?)) // 160), ((||(cnt?)) // 120))  

  delay.pause1ms(2000)
  cogstop(4)                                            ' stop the vga driver cog

  ' vga 320x240 gray scale
  ReadCogFile(string("VGA320.Cog"))                   ' cog 4 
  gray320.bmpenginestart(cogarray, @screen, 2, 2, 320,240)   ' pin group 2, color mode 2
  gray320.displayColor(0, %00000000)         ' color 0 = black
  gray320.displayColor(1, %01010100)         ' color 1 = dark gray
  gray320.displayColor(2, %10101000)         ' color 2 = light gray
  gray320.displayColor(3, %11111100)         ' color 3 = white
  gray320.displayclear(%10101010_10101010_10101010_10101010,@screen) ' 10 is light grey
  'gray320.plotpixel(%%0,10,10,@screen)       ' plot some pixels
  'gray320.plotpixel(%%1,20,10,@screen)   
  repeat i from 0 to 500                                ' put some dots on the screen
      gray320.plotPixel(((||(cnt?)) // 4), ((||(cnt?)) // 320), ((||(cnt?)) // 240), @screen)  
  delay.pause1ms(2000)

  ' temporarily comment this out to save space for testing other cogjects
{
  cogstop(4)
  ' vga 128x64 green on black
  cols := 128
  rows := 64
  ReadCogFile(string("VGA12864.Cog"))
  vga128.start(cogarray, 16, @screen, @colors, @cx0, @sync)      ' start vga driver  
  repeat i from 0 to rows - 1 
    colors[i] := %%0000_0300                              ' green on black

  bytefill(@screen,32,constant(160 * 120))     ' fill screen with spaces

  delay.pause1ms(2000)
  bytemove(@screen.byte[0],string("Hello World"),11)
}
' this code commented out as the program now fills up the memory within a few longs
{  delay.pause1ms(2000)
  cogstop(4)                    ' stop the vga driver cogs
  cogstop(5)
  
  ' vga 100x50 white on blue
  cols := 100
  rows := 50
  ReadCogFile(string("VGA10050.Cog"))
  vga100.start(cogarray, 16, @screen, @colors, @cx0, @sync)      ' start vga driver
  repeat i from 0 to rows - 1
    colors[i] := %%0020_3330                              ' white on blue
  delay.pause1ms(2000)
}  
  cogstop(4)                    ' stop the vga driver cogs
  cogstop(5)
  
  ' vga 80x40 gold on blue
  cols := 80
  rows := 40
  ReadCogFile(string("VGA8040.Cog"))                     ' cog 4 and 5
  vga80.start(cogarray, 16, @screen, @colors, @cx0, @sync)      ' start vga driver
  repeat i from 0 to rows - 1
    colors[i] := %%0020_3300                              ' gold on blue
  bytefill(@screen,32,constant(160 * 120))     ' fill screen with spaces
  bytemove(@screen.byte[0],string("Hello World"),11)
' ************** trig function **************  
  ReadCogFile(string("Trig.Cog"))             ' loads into cog 6
  trig.start(cogarray)                        ' test the trig object, starts in cog6 as this is the first one free
  serial.dec(trig.sine32(4000))
  cogstop(6)                                  ' stop cog 6
' ************* end trig functions **********
  'set cursor 0 to be a solid block
  cm0 := %001
  'set cursor 1 to be a blinking underscore
  cm1 := %111

  ' start the ram driver in cog 6
  ReadCogFile(string("DracRam.Cog"))                     'load the ram driver 
  ram.start(cogarray)                                   ' start the driver
  ram.DoCmd("N",0,0,0)                                  ' turn led on
  delay.pause1ms(100)
  ram.DoCmd("F",0,0,0)                                  ' turn led off
  i := 65                                               ' test with an A
  ram.DoCmd("H",0,0,0)                                  ' set the high latch to 0
  ram.DoCmd("W",@i,100,1)                               ' write value in i to external ram
  i :=0                                                 ' reset i to zero
  ram.DoCmd("R",@i,100,1)                               ' read it back
  bytemove(@screen.byte[20],@i,1)                        ' print on the screen
  
  ' start the mouse in cog 7
  chrs := cols*rows                                     ' number of characters
  ReadCogFile(string("Mouse.Cog"))                      ' read mouse cogject
  mouse.start(cogarray, 24, 25)                         ' and start
  mouse.bound_limits(0, 0, 0, cols - 1, rows - 1, 0)
  mouse.bound_scales(2, -3, 0)

  'main loop - mouse controls stuff
  repeat
    cx0 := mouse.bound_x      'cursor 0 follows mouse
    cy0 := mouse.bound_y
    
    if mouse.button(0)        'if left-click, cursor 1 moves to mouse
      cx1 := mouse.bound_x
      cy1 := mouse.bound_y
      
    if mouse.button(1)        'if right-click, scroll random characters
      'wait for refresh to avoid chopiness during scroll
      repeat until sync       
      sync := 0
      'scroll center of screen
      longmove(@screen + cols * 8, @screen + cols * 9, (rows - 10) * cols >> 2)
      'fill 2nd-to-last line with random characters, mainly spaces
      repeat i from (chrs - cols << 1) to (chrs - cols - 1)
        screen.byte[i] := ?j & (not j >> 28) + " "

 
{  
  ReadCogFile(string("VGA8040.Cog"))                    ' load terminal cog - used twice
  vt100.start(cogarray,16)                             ' start a simple VGA terminal
  
  printstringcr(string("Cogjects demonstration"))
  printstringcr(string("Hit a key"))
  i := kb.getkey
  printchar(i)
  crlf
  hexdump                                             'print out last cog to load
  printstringcr(string("Finished"))
}
PRI ReadCogFile(Filename)
  result := \fat.openFile(Filename, "R")                ' create file and open file
  result := \fat.readdata(cogarray,24)                 ' read first 24 bytes and discard
  result := \fat.readdata(cogarray,fat.filesize-24)    ' read the data into cogarray
  result := \fat.closefile                              ' close the file
{  
PRI PrintStringCR(pstring)' sends pstring to the vga and serial port
     PrintString(pstring)                               ' print the string  
     crlf

PRI PrintString(pstring)' sends pstring to the vga and serial port with no CRLF
    vt100.str(pstring)                                 ' send to vga screen
    'serial.str(pstring)                                 ' send to serial port

PRI PrintChar(c) ' sends c to vga 
    vt100.out(c)                                        ' send to vga
    'serial.tx(c)                                        ' send to serial port
    
PRI CRLF
    PrintChar(13)                                        ' send to vga
    PrintChar(10)                                        ' send to vga

PRI Hexdump | i,j,k,n' print out all the cogject in hex, 32 bytes per line
    k :=0
    repeat i from 0 to 63 ' 2048 bytes is 64 rows x 32 bytes. 
      vt100.hex(k,4)
      vt100.out(32)
      repeat j from 0 to 31
        bytemove(@n,cogarray+k,1)
        vt100.hex(n,2)
        k +=1
      if k == 1024
        crlf
        printstringcr(string("Hit a key")) ' 2 pages
        kb.getkey    
      crlf
} 
{{

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                  TERMS OF USE: MIT License
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
// Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
// COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}}